package org.ow2.util.deployment.annotations.analyzer;

import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.net.URLConnection;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.ow2.easybeans.asm.ClassReader;
import org.ow2.util.deployment.annotations.analyzer.configurator.ArchiveConfigurator;
import org.ow2.util.ee.deploy.api.archive.ArchiveException;
import org.ow2.util.ee.deploy.api.archive.IArchive;

/**
 * ArchiveScanner.
 * @author Gael Lalire
 */
public class ArchiveScanner {

    /**
     * the archive to scan.
     */
    private IArchive archive;

    /**
     * a list of configurator.
     */
    private List<ArchiveConfigurator> archiveConfigurators;

    /**
     * Constructor.
     * @param archive the archive to scan
     */
    public ArchiveScanner(final IArchive archive) {
        this.archive = archive;
        archiveConfigurators = new ArrayList<ArchiveConfigurator>();
    }

    /**
     * add a configurator.
     * @param archiveConfigurator an archive configurator
     */
    public void addArchiveConfigurator(final ArchiveConfigurator archiveConfigurator) {
        archiveConfigurators.add(archiveConfigurator);
    }

    /**
     * remove a configurator.
     * @param archiveConfigurator an archive configurator
     */
    public void removeArchiveConfigurator(final ArchiveConfigurator archiveConfigurator) {
        archiveConfigurators.remove(archiveConfigurator);
    }

    /**
     * begin the scan.
     * @throws AnalyzerException if error occurs
     */
    public final void scan() throws AnalyzerException {
        Iterator<URL> iterator;
        try {
            iterator = archive.getResources();
            // Scan all resources
            while (iterator.hasNext()) {
                URL url = iterator.next();
                // only .class
                if (url.getPath().toLowerCase().endsWith(".class")) {
                    InputStream is = null;
                    try {
                        URLConnection urlConnection = url.openConnection();
                        urlConnection.setDefaultUseCaches(false);
                        is = urlConnection.getInputStream();
                        new ClassReader(is).accept(new ScanClassVisitor(archiveConfigurators), 0);
                    } catch (Exception ioe) {
                        throw new AnalyzerException("Error while analyzing file entry '" + url + "' in archive '"
                                + archive.getName() + "'", ioe);
                    } finally {
                        if (is != null) {
                            try {
                                is.close();
                            } catch (IOException e) {
                                throw new AnalyzerException("Error while closing input stream of the entry '" + url
                                        + "' in archive '" + archive.getName() + "'", e);
                            }
                        }
                    }
                }
            }

        } catch (ArchiveException e) {
             throw new AnalyzerException("Error while analyzing archive '" + archive.getName() + "'", e);
        } finally {
            archive.close();
        }

    }

}
