package org.ow2.util.deployment.annotations.analyzer;

import java.util.ArrayList;
import java.util.List;

import org.ow2.easybeans.asm.AnnotationVisitor;

/**
 * ScanAnnotationVisitor.
 * @author Gael Lalire
 */
public final class ScanAnnotationVisitor implements AnnotationVisitor {

    /**
     * visitors.
     */
    private List<AnnotationVisitor> annotationVisitors;

    /**
     * Constructor.
     * @param annotationVisitors visitors
     */
    public ScanAnnotationVisitor(final List<AnnotationVisitor> annotationVisitors) {
        this.annotationVisitors = annotationVisitors;
    }

    /**
     * visit.
     * @param arg0 ?
     * @param arg1 ?
     */
    public void visit(final String arg0, final Object arg1) {
        for(AnnotationVisitor annotationVisitor : annotationVisitors) {
            annotationVisitor.visit(arg0, arg1);
        }
    }

    /**
     * visitAnnotation.
     * @param arg0 ?
     * @param arg1 ?
     * @return a scanner for all visitors
     */
    public AnnotationVisitor visitAnnotation(final String arg0, final String arg1) {
        List<AnnotationVisitor> result = new ArrayList<AnnotationVisitor>();
        for(AnnotationVisitor annotationVisitor : annotationVisitors) {
            result.add(annotationVisitor.visitAnnotation(arg0, arg1));
        }
        return new ScanAnnotationVisitor(result);
    }

    /**
     * visitArray.
     * @param arg0 ?
     * @return a scanner for all visitors
     */
    public AnnotationVisitor visitArray(final String arg0) {
        List<AnnotationVisitor> result = new ArrayList<AnnotationVisitor>();
        for(AnnotationVisitor annotationVisitor : annotationVisitors) {
            result.add(annotationVisitor.visitArray(arg0));
        }
        return new ScanAnnotationVisitor(result);
    }

    /**
     * visitEnd.
     */
    public void visitEnd() {
        for(AnnotationVisitor annotationVisitor : annotationVisitors) {
            annotationVisitor.visitEnd();
        }
    }

    /**
     * visitEnum.
     * @param arg0 ?
     * @param arg1 ?
     * @param arg2 ?
     */
    public void visitEnum(final String arg0, final String arg1, final String arg2) {
        for(AnnotationVisitor annotationVisitor : annotationVisitors) {
            annotationVisitor.visitEnum(arg0, arg1, arg2);
        }
    }

}
