package org.ow2.util.deployment.annotations.analyzer.configurator;

import java.util.HashMap;
import java.util.Map;

import org.ow2.easybeans.asm.AnnotationVisitor;
import org.ow2.easybeans.asm.commons.EmptyVisitor;
import org.ow2.util.deployment.metadata.structures.JAnnotation;

/**
 * Implements a generic getAnnotationVisitor method with map.
 * @author Gael Lalire
 */
public abstract class BasicConfigurator implements CommonConfigurator {

    /**
     * a empty annotation visitor.
     */
    private static final AnnotationVisitor EMPTY_ANNOTATION_VISITOR = new EmptyVisitor();

    /**
     * map a annotation class name with a visitor.
     */
    private Map<String, AnnotationVisitor> annotationVisitors;

    /**
     * Constructor.
     */
    public BasicConfigurator() {
        annotationVisitors = new HashMap<String, AnnotationVisitor>();
    }

    /**
     * @return map where add visitor
     */
    protected Map<String, AnnotationVisitor> getAnnotationVisitors() {
        return annotationVisitors;
    }

    /**
     * utility method which avoid creation of multiple empty visitor.
     * @return a empty visitor
     */
    protected final AnnotationVisitor getEmptyAnnotationVisitor() {
        return EMPTY_ANNOTATION_VISITOR;
    }

    /**
     * implementation of common configurator method.
     * @param annotation signature of annotation
     * @return the visitor in the map or empty visitor
     */
    public final AnnotationVisitor getAnnotationVisitor(final JAnnotation annotation) {
        AnnotationVisitor annotationVisitor = annotationVisitors.get(annotation.getName());
        if(annotationVisitor == null) {
            return getEmptyAnnotationVisitor();
        }
        return annotationVisitor;
    }

    /**
     * implementation of common configurator method.
     * This one do nothing, you may override it.
     */
    public void configurationComplete() {
    }

}
