package org.ow2.util.deployment.metadata.structures;

/**
 * Annotation on class, method or field.
 * @author Gael Lalire
 */
public final class JAnnotation {

    /**
     * annotation name.
     */
    private String name;

    /**
     * visibility of annotation.
     */
    private boolean visible;

    /**
     * Constructor.
     * @param name name
     * @param visible visible
     */
    public JAnnotation(final String name, final boolean visible) {
        if(name == null) {
            throw new IllegalArgumentException("Name argument of JAnnotation constructor can not be null");
        }
        this.name = name;
        this.visible = visible;
    }

    /**
     * @return visibility of annotation
     */
    public boolean isVisible() {
        return visible;
    }

    /**
     * @return name of annotation
     */
    public String getName() {
        return name;
    }

    /**
     * @return name hashcode
     */
    @Override
    public int hashCode() {
        return name.hashCode();
    }

    /**
     * @param obj the object to compare
     * @return true if same annotation.
     */
    @Override
    public boolean equals(final Object obj) {
        if(obj==this) {
            return true;
        }
        if(!(obj instanceof JAnnotation)) {
            return false;
        } else {
            JAnnotation jAnnotation = (JAnnotation) obj;
            if(name.equals(jAnnotation.name) && visible==jAnnotation.visible) {
                return true;
            }
            return false;
        }
    }

}
